import '../contexts/context.dart';
import '../contexts/result.dart';
import '../parser.dart';

/// Returns a parser that succeeds at the end of input.
Parser<void> endOfInput([String message = 'end of input expected']) =>
    EndOfInputParser(message);

/// A parser that succeeds at the end of input.
class EndOfInputParser extends Parser<void> {
  final String message;

  EndOfInputParser(this.message)
      : assert(message != null, 'message must not be null');

  @override
  Result parseOn(Context context) => context.position < context.buffer.length
      ? context.failure(message)
      : context.success(null);

  @override
  int fastParseOn(String buffer, int position) =>
      position < buffer.length ? -1 : position;

  @override
  String toString() => '${super.toString()}[$message]';

  @override
  EndOfInputParser copy() => EndOfInputParser(message);

  @override
  bool hasEqualProperties(EndOfInputParser other) =>
      super.hasEqualProperties(other) && message == other.message;
}
